/*!
* @file LPC_Include.h
*
* @brief Costanti e tipi principali usati da tutti i moduli del compito di LPC
*
* Questo file contiene valori costanti comuni a tutti
* i moduli del compito di LPC e facenti parte dell'interfaccia.
*
* Contiene anche la definizione delle strutture da utilizzare e i prototipi delle
* funzioni dell'interfaccia.
*
* @author Stefano Guerrini
* @author Andrea Sterbini
* @author Gianni Campanile
* @date	  Gennaio 2006
* @version 1.0
*********************************************************************/

#ifndef _LPC_INCLUDE_INCLUDED
#define _LPC_INCLUDE_INCLUDED

#include <netinet/in.h>

#include <stdio.h>
#include <time.h>
#include <errno.h>

/*!
*	\addtogroup ComuniLPC Specifiche comuni a tutti i Moduli
*	\{
*/

/********** COSTANTI DI LPC ************/
/** @defgroup CostantiLPC Costanti
*
* Costanti comuni a tutti i moduli 
* @{
*/
// Valori TRUE e FALSE se non gia' definiti
#ifndef TRUE
#define TRUE 1	//!< Valore TRUE 
#define FALSE 0	//!< Valore FALSE
#endif

// Alcuni limiti del sistema
#define DBF_NAME_LENGTH		10	//!< Lunghezza massima del nome di un database file
#define FIELD_NAME_LENGTH	10	//!< Lunghezza massima del nome di un campo
#define INDEX_NAME_LENGTH	10	//!< Lunghezza massima del nome di un index file
#define MAX_NUM_FIELDS		255	//!< Numero massimo di campi nel database
#define MAX_DBF_OPEN		32	//!< Numero massimo di file di database aperti contemporaneamente
#define MAX_INDEX_OPEN		10	//!< Numero massimo di indici creabili per un database file

// Costanti per il tipo di campo e per la sua lunghezza
#define CHARACTER	1 //!< Tipo campo CHARACTER 
#define NUMERIC		2 //!< Tipo campo NUMERIC 
#define DATE		3 //!< Tipo campo DATE 
#define LOGICAL		4 //!< Tipo campo LOGICAL

// Estensioni dei file e lunghezza totale nome
#define DBF_EXTENSION	".dbf"	//!< Estensione del nome di un database file
#define DBF_OPEN_EXT	".$$$"	//!< Estensione di un database aperto
#define DBF_FILENAME_LENGTH	DBF_NAME_LENGTH+1+3	//!< Lunghezza totale del nome di un file DBF compresa l'estensione

// Lunghezze dell'header
#define DBF_FIXED_HEADER_LEN 	(DBF_NAME_LENGTH + 1 + sizeof(short) + sizeof(long) + sizeof(short) + sizeof(short) + sizeof(long))  //!< Lunghezza della parte fissa dell'header escluso il doppio NULL finale
#define DBF_FIELD_LEN 		(FIELD_NAME_LENGTH + 1 + sizeof(short) + sizeof(short))  //!< Lunghezza di ogni definizione campo

// Costanti per la lunghezza di ogni tipo campo (massima)
#define FIELD_CHAR_LEN	255 //!< Lunghezza massima del tipo campo CHARACTER 
#define FIELD_NUM_LEN	sizeof(double)	//!< Lunghezza (FISSA) del tipo campo NUMERIC
#define FIELD_DATE_LEN	sizeof(long)	//!< Lunghezza (FISSA) del tipo campo DATE
#define FIELD_LOGI_LEN	sizeof(char)	//!< Lunghezza (FISSA) del tipo campo LOGICAL 

// Costanti per il flag di inserimento record
#define TOP		1 //!< Inserimento all'inizio del file 
#define BOTTOM	2 //!< Inserimento alla fine del file
#define CURRENT	3 //!< Inserimento nel punto corrente del file

// Modalita' di spostamento su un DBF
#define ABSOLUTE	1 //!< Spostamento assoluto 
#define RELATIVE	2 //!< Spostamento relativo

// definizione delle costanti per gli errori
#define	LPC_OK 			0	//!< Tutto bene
#define LPC_BAD_ARG		-1	//!< Argomento non valido
#define LPC_BAD_FIELD 	-2	//!< Campo non valido
#define LPC_FILE_EXISTS -3	//!< File gia' esistente
#define LPC_FILE_ACCESS -4	//!< Accesso negato o file inesistente
#define LPC_ERR_WRITE	-5	//!< Errore di scrittura
#define LPC_ERR_READ	-6	//!< Errore di lettura
#define LPC_ERR_SEEK	-7	//!< Errore di posizionamento
#define LPC_NO_MEMORY	-8	//!< Errore di allocazione memoria
#define LPC_NO_SPACE	-9	//!< Spazio interno esaurito (vettore full)
#define LPC_DBF_OPEN	-10	//!< Il database  aperto 
#define LPC_NOT_FOUND	-11 //!< Record non trovato
#define LPC_ERR_EOF		-12 //!< Errore di fine file


/** @} */	// Fine gruppo costanti

/********** STRUTTURE DI LPC ************/
/** @defgroup StruttureLPC Strutture
*
* Strutture comuni a tutti i moduli 
* @{
*/


//! Definizione di un campo del database
typedef struct {
	
	char	Name[FIELD_NAME_LENGTH+1];	//!< Nome del campo
	short		FieldType;	//!< Tipo di campo
	short		FieldLen;	//!< Lunghezza del campo. Se numerico, lunghezza della parte intera.
	short		FieldDecs;	//!< Decimali del campo (se ve ne sono, altrimenti 0)
	
} FIELD_DATA_t;


//! Informazioni su un database. Sono informazioni generiche pi informazioni dipendenti dalla connessione
typedef struct {
	
	FILE	*fp;		//!< File pointer del database aperto
	char	Name[DBF_NAME_LENGTH+1];	//!< Nome del database escluso estensione
	int		NumFields;	//!< Numero dei campi del database
	long	NumRecs;	//!< Numero totale di records nel database
	int		HeaderLen;	//!< Lunghezza dell'Header
	int		RecordLen;	//!< Lunghezza di un record del database
	long	CurrentRec;	//!< Numero del record corrente 
	time_t	LastUpdate;	//!< Data e ora ultimo aggiornamento (Julian date)
	
} DBF_INFO_t;

//! Valore di un campo del database
typedef struct {
	
	char	Name[FIELD_NAME_LENGTH+1];	//!< Nome del campo
	void	*value;
	
} FIELD_VALUE_t;


/** @} */	// Fine gruppo strutture

// Non voglio includere documentazione sui prototipi


/********** FUNZIONI DI LPC ************/
/* FunzioniLPC  Prototipi funzioni di interfaccia
*
* Funzioni di interfaccia 
*/
// Funzione di gestione dell'errore
extern void LPC_GestioneErrore(int ErrorCode, char *Func, char *Msg);

/************ Modulo 1 **/
//Crea un DBF
extern int CreateDatabaseFile(char *Name, int NumFields, FIELD_DATA_t *FieldData);

// Cancella un DBF
extern int DeleteDatabaseFile(char *Name);


/************ Modulo 2 **/
// Apre un DBF
extern int OpenDatabaseFile(char *Name);

// Chiude un DBF
extern int CloseDatabaseFile(int handle);

// Legge informazioni di un DBF aperto
extern int GetDBFInfo(int handle, DBF_INFO_t *DBFInfo);

// Legge informazioni di uno o piu' campi di un DBF aperto
extern int GetFieldInfo(int handle, int NumFields, FIELD_DATA_t *FieldData);

// Verifica se un handle  corretto
extern int IsDBFHandle(int handle);

/************ Modulo 3 **/
// Inserisce fisicamente un record vuoto in un DBF
extern int InsertBlankRecord(int handle, int Where);

// Cancella fisicamente il record corrente da un DBF
extern int DeleteRecord(int handle);

// Va ad un record specifico
extern int GotoRecord(int handle, int Mode, long NumRec);


/************ Modulo 4 **/

// Legge un record di un DBF
extern int ReadRecord(int handle, FIELD_VALUE_t *FieldValue);


// Modifica i valori del record corrente
extern int EditRecord(int handle, int NumFields, FIELD_VALUE_t *FieldValue);


// Cerca un record con un campo con un valore definito
extern int LocateRecord(int handle, FIELD_VALUE_t *FieldValue, int Where);


/*! \} */	// Fine gruppo Comuni


#endif // _LPC_INCLUDE_INCLUDED
